% loudspeaker.m
% Loudspeaker BEM acoustics demo

clear;

% Import the mesh and its parameters
speaker = gmsh_import_mesh('../meshes/loudspeaker.msh');
params = gmsh_import_params('../meshes/loudspeaker.geo');
% Create field mesh
field = mesh_create_slab([0 -1 -1; 0 1 -1; 0 1 1; 0 -1 1], [100 100]);

field = mesh_section(field, [-inf -params.y_m-5e-3 -params.l_z-5e-3;
        inf, params.l_y-params.y_m+5e-3 5e-3], 'none');

% Plot the surface and field meshes
figure;
plot_mesh(speaker); hold on;
plot_mesh(field);

%% Determine maximal frequency
c = 343;            % Speed of sound [m/s]
rho0 = 1.2;         % Average density [kg/m^3]
% At least 6 elems / lambda
k_max = mesh_kmax(speaker, 6);  
f_max = k_max * c / (2*pi);
% Plot the maximal frequency / element over the mesh
figure;
plot_mesh(speaker, f_max); axis equal;
cb = colorbar;
ylabel(cb, 'Max freq. per elem [Hz]');

%% Define excitation
[cnt, nrm] = mesh_element_props(speaker);
% Select the membrane elements
sel = (cnt(:,1).^2 + cnt(:,2).^2) < params.r_out^2 & ...
    cnt(:,3) > -params.d_m-5e-3;
n_elements = size(speaker.elements, 1);     % Number of elements
a = zeros(n_elements, 3);
a(sel, 3) = 100;                            % Acceleration in z [m/s^2]
an = dot(a, nrm, 2);                        % Normal acceleration

figure;
plot_mesh(speaker, an); axis equal;
cb = colorbar;
ylabel(cb, 'Membrane normal acceleration [m/s^2]', 'FontSize', 12);


%% Solve the BEM system and compute the radiated field
f = 3500;
k = 2*pi*f / c;
fprintf('BEM surface assembly ... '); tic;
[Gs, Hs] = bem_matrices(k, speaker);
fprintf('Ready in %.2f s\n', toc);

% dp/dn on the surface
qs = - rho0 * an;

% Solve the surface equations
fprintf('Solving boundary equation ... '); tic;
ps = (Hs - 0.5*eye(size(Hs, 1))) \ (Gs * qs);
fprintf('Ready in %.2f s\n', toc);

fprintf('BEM field assembly ... '); tic;
[Gf, Hf] = bem_matrices(k, speaker, field.nodes);
fprintf('Ready in %.2f s\n', toc);

% Calculate pressure in field
fprintf('Computing field ... '); tic;
pf = Hf*ps - Gf*qs;
fprintf('Ready in %.2f s\n', toc);
    
%% Rotation before plot
rot = [1 0 0; 0 0 1; 0 -1 0];
speaker.nodes = speaker.nodes*rot;
field.nodes = field.nodes*rot;

%% Plot the solution
figure;
plot_mesh(speaker, real(ps));
hold on
h = plot_mesh(field, real(pf));
set(h, 'EdgeColor', 'none');
caxis([-1 1]);
axis off;
ylim(.5*[-2 1]);
zlim(.4*[-1 1.2]);
view([-45 10]);

cb = colorbar('SouthOutside');
ylabel(cb, 'Pressure field Re(p) [Pa]', 'FontSize', 12);
xlabel('x [m]');
zlabel('z [m]');
