% loudspeaker.m
% Loudspeaker BEM acoustics demo

clear;

% Import the mesh and its parameters
speaker = gmsh_import_mesh('../meshes/loudspeaker_coarse.msh');
params = gmsh_import_params('../meshes/loudspeaker_coarse.geo');

% Create field points
r_field = 2;
n_field = 721;              % Number of field points
phi_field = linspace(-pi, pi, n_field).';
field_crd = [ ...
    % Horizontal plane
    [r_field*sin(phi_field), zeros(size(phi_field)), r_field*cos(phi_field)]
    % Vertical plane
    [zeros(size(phi_field)), r_field*sin(phi_field), r_field*cos(phi_field)]
];

% Plot the surface and field meshes
figure;
plot_mesh(speaker); hold on;
plot3(field_crd(:,1), field_crd(:,2), field_crd(:,3), 'b*');

%% Determine maximal frequency
c = 343;            % Speed of sound [m/s]
rho0 = 1.2;         % Average density [kg/m^3]
% At least 6 elems / lambda
k_max = mesh_kmax(speaker, 6);  
f_max = k_max * c / (2*pi);
% Plot the maximal frequency / element over the mesh
figure;
plot_mesh(speaker, f_max); axis equal;
cb = colorbar;
ylabel(cb, 'Max freq. per elem [Hz]');

%% Define excitation
[cnt, nrm] = mesh_element_props(speaker);
% Select the membrane elements
sel = (cnt(:,1).^2 + cnt(:,2).^2) < params.r_out^2 & ...
    cnt(:,3) > -params.d_m-5e-3;
n_elements = size(speaker.elements, 1);     % Number of elements
a = zeros(n_elements, 3);
a(sel, 3) = 100;                            % Acceleration in z [m/s^2]
an = dot(a, nrm, 2);                        % Normal acceleration

figure;
plot_mesh(speaker, an); axis equal;
cb = colorbar;
ylabel(cb, 'Membrane normal acceleration [m/s^2]', 'FontSize', 12);

%% Create a frequency vector
f_min = 100;
f_top = min(f_max);
n_freq = 3;
f_vec = [100 1000 2000];

% Preallocate for field solution
pf = nan(2*n_field, n_freq);

%% Solve the BEM system and compute the radiated field
% (Repeat the same steps for all the frequencies
for i_f = 1 : n_freq
    f = f_vec(i_f);
    k = 2*pi*f / c;
    
    fprintf('Calculating at %.0f Hz (%d of %d)\n', f, i_f, n_freq);
    
    fprintf('BEM surface assembly ... '); tic;
    [Gs, Hs] = bem_matrices(k, speaker);
    fprintf('Ready in %.2f s\n', toc);

    % dp/dn on the surface
    qs = - rho0 * an;

    % Solve the surface equations
    fprintf('Solving boundary equation ... '); tic;
    ps = (Hs - 0.5*eye(size(Hs, 1))) \ (Gs * qs);
    fprintf('Ready in %.2f s\n', toc);

    fprintf('BEM field assembly ... '); tic;
    [Gf, Hf] = bem_matrices(k, speaker, field_crd);
    fprintf('Ready in %.2f s\n', toc);

    % Calculate pressure in field
    fprintf('Computing field ... '); tic;
    pf(:, i_f) = Hf*ps - Gf*qs;
    fprintf('Ready in %.2f s\n', toc);
end
%% Plot the solution on the surface
figure;
plot_mesh(speaker, real(ps));
hold on
axis off;
caxis(2.5*[-1 1]);
cb = colorbar('SouthOutside');
ylabel(cb, 'Surface pressure Re(p) [Pa]', 'FontSize', 12);
xlabel('x [m]');
zlabel('z [m]');

%% Plot the directivities
figure;
% Horizontal
subplot(1,2,1);
plot(180/pi*phi_field, 20*log10(abs(pf(1:n_field,:)/2e-5)), ...
    'LineWidth', 1.2);
ylim([50 100]);
xlim(180*[-1 1]);
xlabel('\theta [^\circ]');
set(gca, 'XTick', -180 : 90 : 180);
title('Horizontal');
legend(strcat(num2str(round(f_vec(:))), {' Hz'}), 'Location', 'NorthEast');
% Vertical
subplot(1,2,2);
plot(180/pi*phi_field, 20*log10(abs(pf(n_field + (1:n_field),:)/2e-5)), ...
    'LineWidth', 1.2);
ylim([50 100]);
xlim(180*[-1 1]);
xlabel('\phi [^\circ]');
set(gca, 'XTick', -180 : 90 : 180);
title('Vertical');
legend(strcat(num2str(round(f_vec(:))), {' Hz'}), 'Location', 'NorthEast');