function [G, H] = bem_matrices(k, mesh, points)
%BEM_MATRICES Compute the BEM matrices using numerical integration
%   [G, H] = BEM_MATRICES(K, MESH) Computes the surface BEM matrices G and
%       H over the mesh MESH at the wave number K. If the number of
%       elements in the mes is E, then the matrices G and H are of type
%       E x E.
%   [G, H] = BEM_MATRICES(K, MESH, POINTS) Computes the field BEM matrices
%       G and H representing the radiation from the mesh MESH to the field
%       points POINTS. The field points must be given as an F x 3 matrix, 
%       where F is the number of field points. If the mesh has E elements,
%       then the resulting matrices are of type F x E.
%
%   The method computes the BEM matrices from the mesh to the mesh itself
%   or arbitrary collocation points in the field. The computation is
%   performed by cycling the collocation point through the points of the 
%   mesh (element centers) or the field points and computing the effect of
%   all the elements (i.e., one row of the system matrices) at once.
%   In case of surface matrices, singular integrals are handled in a second
%   cycle.

% If no points given
is_surface = (nargin < 3);
    
if (is_surface)
    % Compute element centers (collocation points)
    points = mesh_element_props(mesh);
    % Compute relative distance matrix
    [~, R] = mesh_distances(mesh);
else
    % Compute relative distance matrix
    [~, R] = mesh_distances(mesh, points);
end

n_points = size(points, 1);             % Number of collocation points
n_elements = size(mesh.elements, 1);    % Number of elements in the mesh

% Preallocate for the matrices 
G = zeros(n_points, n_elements);
H = zeros(n_points, n_elements);

% Go through the collocation points
for i = 1 : n_points
    % Set the receiver point as the i-th point
    x0 = points(i, :);
    % Compute quadrature orders based on relative distance
    orders = bem_distance_to_order(k*R(i,:).');
    % Do not calculate singular
    if (is_surface)
        orders(i) = nan;
    end
    % Create the quadrature points over the mesh
    [x, nx, W] = mesh_quadrature_points(mesh, orders);
    % Evaluate the fundamental solution between the receiver collocational 
    % point and the source quadrature points
    [F, dFdn] = fundamental_solution(k, x0, x, nx);
    % The integral is attained as a weighted sum
    G(i, :) = (W * F).';
    H(i, :) = (W * dFdn).';
    % Indicate progress
    misc_progbar(1, n_points, i);
end

% Singular integration as needed
if (is_surface)
    for i = 1 : n_points
        % Calculate singular quadrature
        corners = mesh.nodes(mesh.elements(i, :).', :); 
        [xx, ww] = elem_singular_quadrature(corners, 7);
        F = fundamental_solution(k, points(i, :), xx);
        % Integrate the fundamental solution
        G(i, i) = ww.' * F;
        % The integral of dF/dn is zero over a planar element
        H(i, i) = 0;
        % Indicate progress
        misc_progbar(1, n_points, i);
    end
end % of surface singular integrals

end % of function bem_matrices
