function [xx, ww] = elem_singular_quadrature(corners, order)
%ELEM_SINGULAR_QUAD Generate quadrature for singular integration 
%   [XX, WW] = ELEM_SINGULAR_QUADRATURE(CORNERS, ORDER) Generates the base
%       points XX and weights WW of a quadrature that can integrate the 
%       singular fundamental solution over a triangle. CORNERS specify the
%       corners of the element, each row containing the coordinates of one 
%       corner point. The singular quadrature is generated by means of the 
%       polar Duffy transform of regular quadratures with order ORDER. It 
%       is assumed that the singular point is at the center of the element.

n_corners = size(corners, 1);   % Number of element corners
center = mean(corners, 1);      % Triangle center

n_quad = ceil((order+1)/2)^2;   % Number of quadrature points

% Preallocate for the resulting points
xx = nan(n_corners * n_quad, size(corners, 2));
ww = nan(n_corners * n_quad, 1);

% Subdivide the element into triangles
for i = 1 : n_corners
    tria = [center; corners([i, mod(i, n_corners)+1], :)];
    [x, w] = gaussian_quadrature_2d(order, 3);
    % Map the quadrature points to the new triangle
    N = elem_shape_functions(x, 3);
    x = N * tria;

    % Find the size of the element
    A = norm(cross(tria(2,:) - tria(1,:), tria(3,:) - tria(1,:), 2))/2;
   
    % Collect the quadrature points
    idx = (i-1)*n_quad + (1 : n_quad);
    xx(idx, :) = x;
    % Take the element size into account in weighting
    ww(idx, :) = 2*A*w;         
end

end % of fucnction elem_singular_quadrature
