function [F, dFdn] = fundamental_solution(k, x0, x, nx)
%FUNDAMENTAL_SOLUTION Fundamental solution of the Helmholtz equation in 3D
%   F = FUNDAMENTAL_SOLUATION(K, X0, X) Computes the fundamental solution
%       of the 3D Helmholtz equation with the wave number K between the
%       receiver point X0 and the source points X. If X has N rows, then 
%       the result F is a column vector having N elements.
%   [F, DFDN] = FUNDAMENTAL_SOLUTION(K, X0, X, NX) Also computes the normal
%       derivative of the fundamental solution with respect to X. In this
%       case the normal vectors NX at the source points X must also be
%       specified.


% Compute the distance
d = bsxfun(@minus, x, x0);
r = sqrt(dot(d, d, 2));

% Compute the fundamental solution 
F = exp(-1i*k*r) ./ (4*pi*r);

% Compute the normal derivative as needed
if nargout > 1
    rdnx = dot(d, nx, 2) ./ r;
    dFdn = -exp(-1i*k*r) ./ r.^2 .* (1 + 1i*k*r) .* rdnx / (4*pi);
end

end % of function fundamental_solution

