function [xx, ww] = gaussian_quadrature_2d(order, nvert)
%GAUSSIAN_QUADRATURE_2D Create 2D Gaussian quadrature integration
%   [X, W] = GAUSSQUAD2(ORDER, NVERT) returns Gaussian quadrature base
%       points X and weights W for numerical integration over stadard 2D 
%       element domains.
%   
% Input parameters:
%   ORDER : Polynomial order of the quadrature
%   NVERT : Number of vertices. 3 for TRIA, 4 for QUAD elements.
%           The standard TRIA element's coordinates are
%           [(0,0), (+1,0), (0,+1)]
%           The standard QUAD's coordinates are
%           [(-1,-1), (+1,-1), (+1,+1), (-1,+1)]
% Output:          
%   X     : Gaussian quadrature base points
%   W     : Gaussian quadrature weights

% Create qudrature based on the number of vertices
switch nvert
    case 3  % TRIA
        % Compute over the quadrangle
        [xi, w] = gaussian_quadrature_2d(order, 4);
        % Transform over the triangle
        [xx, ww] = elem_duffy_transform(xi, w);
    case 4  % QUAD
        n = ceil((order+1)/2);
        [xi, w] = gaussian_quadrature(n);   % Gauss quadrature in (-1,+1)
        [eta, xi] = meshgrid(xi,xi);        % Base points over unit square
        w = (w * w.');                      % Weights over the square
        xx = [xi(:) eta(:)];
        ww = w(:);
    otherwise
        error('gaussian_quadrature_2d:invalid_argument',...
            'Input argument NVERT should be 3 for TRIA and or 4 for QUAD');
end
end % of gaussian_quadrature_2d
