function [mesh, info] = gmsh_import_mesh(fname)
%GMSH_IMPORT_MESH Load a triangular mesh from a GMSH mesh file
%   [MESH, INFO] = GMSH_IMPORT_MESH(FILENAME) Loads a gmsh mesh file from
%       the file specified by FILENAME. Output MESH will contain the nodes
%       and triangular elements of the mesh. INFO will contain the
%       information fields read from the mesh file.
%
%   Notes:
%       - This function is compatible with gmsh meshes created by gmsh
%         versions 2.x and 3.x. The new mesh format introduced in versions
%         4.x is not supported yet.
%       - The function only loads triangular elements. Other element types
%         in the mesh are not loaded. Nodes not used by the triangular
%         elements are automatically dropped.

fid = fopen(fname);
if (fid < 0)
    error('gmsh_import_mesh:file_error', ...
        'Could not open input file: %s', fname);
end
ftext = textscan(fid, '%s', 'Delimiter', '\n');
fclose(fid);

% Commands start with $ sign
rxp_command = '^\$';

r = regexp(ftext{1}, rxp_command, 'start');

k = 1;
commands = cell(0,2);

for i = 1 : numel(r)
    if ~isempty(r{i})
        commands{k, 1} = ftext{1}{i};
        commands{k, 2} = i;
        k = k+1;
    end
end

% Obtain info
info = read_info(ftext{1}, commands);

% Read nodes
nodes = read_nodes(ftext{1}, commands);

% Read elements
elements = read_elements(ftext{1}, commands);

node_keep = ismember(nodes(:,1), elements);
nodes = nodes(node_keep, :);

% Drop unused nodes and renumber as needed
node_ids = nodes(:, 1);

node_inds(node_ids+1) = (1:size(nodes,1))+1;
node_inds(1) = 1;

elements = node_inds(elements+1)-1;

mesh.nodes = nodes(:, 2:4);
mesh.elements = elements;

end % of function gmsh_import_mesh

function info = read_info(ftext, commands)
    start_exp = '^\$MeshFormat';
    %stop_exp = '^\$EndMeshFormat';
    
    start_ind = regexp(commands(:,1), start_exp, 'start', 'once');
    start_ind = find(~cellfun(@isempty, start_ind), 1, 'first');
    
    % If no info found
    if isempty(start_ind)
        info = [];
        return
    end
    
    % Starting line of information
    start_line = ftext{commands{start_ind,2}+1};
    
    % Process the structure
    data = sscanf(start_line, '%f', 3);
    
    info.version = data(1);
    if data(2) == 0
        info.file_type = 'ASCII';
    else
        info.file_type = data(2);
    end
    info.data_size = data(3);
end

function nodes = read_nodes(ftext, commands)
    start_exp = '^\$Nodes';
    stop_exp = '^\$EndNodes';
    
    start_ind = regexp(commands(:,1), start_exp, 'start', 'once');
    start_ind = find(~cellfun(@isempty, start_ind), 1, 'first');
    
    stop_ind = regexp(commands(:,1), stop_exp, 'start', 'once');
    stop_ind = find(~cellfun(@isempty, stop_ind), 1, 'first');
    
    % If no info found
    if isempty(start_ind) || isempty(stop_ind)
        nodes = [];
        return
    end
    
    start_line_ind = commands{start_ind,2}+2;
    stop_line_ind = commands{stop_ind,2}-1;
    
    nNodes = stop_line_ind - start_line_ind + 1;
    nodes = zeros(nNodes, 4);
    
    % ElemID, x, y, z format
    for iNode = 1 : nNodes
        nodes(iNode, :) = sscanf(ftext{start_line_ind + iNode -1}, '%f', 4);
    end
end

function elements = read_elements(ftext, commands)
    start_exp = '^\$Elements';
    stop_exp = '^\$EndElements';
    
    start_ind = regexp(commands(:,1), start_exp, 'start', 'once');
    start_ind = find(~cellfun(@isempty, start_ind), 1, 'first');
    
    stop_ind = regexp(commands(:,1), stop_exp, 'start', 'once');
    stop_ind = find(~cellfun(@isempty, stop_ind), 1, 'first');
    
    % If no info found
    if isempty(start_ind) || isempty(stop_ind)
        elements = [];
        return
    end
    
    start_line_ind = commands{start_ind,2}+2;
    stop_line_ind = commands{stop_ind,2}-1;
    
    nElem = stop_line_ind - start_line_ind + 1;
    elements = zeros(nElem, 3);
    jElem = 0;
    for iElem = 1 : nElem
        data = sscanf(ftext{start_line_ind + iElem - 1}, '%f', Inf);
        if (numel(data) < 2)
            continue
        end
        % Select elem type
        switch data(2)
            % 3-node tria
            case 2
                elements(jElem+1, :) = data(end-2 : end);
                jElem = jElem+1;
            otherwise
                % Nothing to do here
        end
    end
    % Drop unread elements
    elements = elements(1 : jElem, :);
end
