function [p, q] = incident_field(type, varargin)
%INCIDENT_FIELD Calculate incident wave field from simple sources
%   [P, Q] = incident('point', R0, R, N, K) Computes the field generated by
%       point sources located at R0 at the locations R.
%   [P, Q] = incident('line', R0, R, N, K) Computes the field generated by
%       line sources located at R0 at the locations R.
%   [P, Q] = incident('plane', DIR, R, N, K) Computes the field of plane
%       waves with propagation directions given by DIR at the locations R.
%
%   In each case, R0 are the source locations given in an M x 3 matrix for
%   the M sources. In case of 'plane' type, DIR are the directions in an 
%   M x 3 matrix. R are the locations where the field is computed, given in
%   an N x 3 matrix. N is the normal direction at R, also given in an N x 3
%   matrix. N is required only for computing the normal derivative of the
%   pressure field. If the normal derivative is not needed, N is not used
%   and can be empty. K is the scalar wave number.
%
%   The output P and Q are the pressure field and its normal derivatives,
%   respectively, given as an N x M matrix, each column containing the
%   field of one of the M sources at all locations.

r = varargin{2};
n = varargin{3};
k = varargin{4};
N = size(r,1);

switch lower(type)
    case 'point'
        r0 = varargin{1};
        M = size(r0,1);
        p = zeros(N,M);
        if nargout == 2
            q = zeros(N,M);
        end
        for m = 1 : M
            dvec = bsxfun(@minus, r, r0(m,:));
            d = sqrt(dot(dvec, dvec, 2));
            p(:,m) = p(:,m) + exp(-1i*k*d)./ d / (4*pi);
            if nargout == 2
                rdn = dot(dvec, n, 2) ./ d;
                q(:,m) = q(:,m) -(1+1i*k*d)./d .* p(:,m) .* rdn;
            end
        end
    case 'line'
        r0 = varargin{1};
        M = size(r0,1);

        p = zeros(N,M);
        if nargout == 2 
            q = zeros(N,M);
        end
        for m = 1 : M
            dvec = r(:,1:2) - repmat(r0(m,1:2), N, 1);
            d = sqrt(dot(dvec, dvec, 2));
            p(:,m) = p(:,m) -1i/4 * besselh(0,2,k*d);
            if nargout == 2
                rdn = dot(dvec(:,1:2), n(:,1:2), 2) ./ d;
                q(:,m) = q(:,m) + 1i*k/4 * besselh(1,2,k*d) .* rdn;
            end
        end
    case 'plane'
        dir = varargin{1};
        M = size(dir, 1);
        p = zeros(N, M);
        if nargout == 2
            q = zeros(N, M);
        end
        for m = 1 : M
            dirm = dir(m,:)/  norm(dir(m,:));
            kk = dirm * k;
            p(:, m) = exp(-1i*(r*kk(:)));
            if nargout == 2
                q(:,m) = -1i*(n*kk(:)) .* p(:,m);
            end
        end
    otherwise
        error('incident_field:invalid_argument', ...
            'Could not interpret source type %s', type);
end % of switch

end % of function incident_field
