function mesh = mesh_create_slab(corners, n_elems)
%MESH_CREATE_SLAB Create the mesh of a quadrangle slab
%   MESH = MESH_CREATE_SLAB(CORNERS, N_ELEMS) Creates a slab with the four
%       corner points located at CORNERS and the number elements along two
%       neighboring edges given by N_ELEMS. The resulting mesh MESH will
%       contain triangular elements.
%
%       CORNERS must be a 4 x 2 (2D mesh) or 4 x 3 (3D mesh) matrix. The 
%       corners follow each other in the order of connection.
%
%       N_ELEMS can be a scalar value, in this case the slab is divided
%       into N_ELEMS elements along each side. N_ELEMS can also be a
%       two-dimensional vector, then the slab is divided into N_ELEMS(1) x
%       N_ELMES(2) elements.

if (size(corners, 1) ~= 4)
    error('mesh_create_slab:invalid_argument', ...
        'Input parameter CORNERS must have exactly 4 rows');
end

if (size(corners, 2) < 3)
    corners(end, 3) = 0;
end

if (length(n_elems) == 1)
    n_elems = [n_elems n_elems];
end

% Create a grid in the reference domain [-1, 1] x [-1, 1]
xi = linspace(-1, 1, n_elems(1)+1);
eta = linspace(-1, 1, n_elems(2)+1);
[Xi, Eta] = ndgrid(xi(:), eta(:));

% Project the reference grid using the corner cordinates
N = elem_shape_functions([Xi(:), Eta(:)], 4);
mesh.nodes = N * corners;

% Compute the number of elements (there will be two times this many trias)
n_elem = prod(n_elems);
elem = (0 : n_elem-1).';

% Row and column indices for all elements
row_index = floor(elem/n_elems(1));
col_index = mod(elem, n_elems(1));

% The base node index of the first triangle
base_index = row_index*(n_elems(1)+1) + col_index+1;

% Fill elements
mesh.elements = zeros(2*n_elem, 3);
mesh.elements(1 : 2 : end, :) = ...
    bsxfun(@plus, base_index, [0, 1, n_elems(1)+1]);
mesh.elements(2 : 2 : end, :) = ...
    bsxfun(@plus, base_index, [1, n_elems(1)+2, n_elems(1)+1]);

end % of function mesh_create_slab
