function [d, r] = mesh_distances(mesh, points)
%MESH_DISTANCES Compute the distance between mesh and field points
%   [D, R] = MESH_DISTANCES(MESH) Computes the distance of the element
%       centers of a triangular mesh given in MESH. The resulting matrices
%       are of type E x E, with E denoting the number of elements in the
%       mesh.
%   [D, R] = MESH_DISTANCES(MESH, POINTS) Computes the distance of the
%       element centers of a triangular mesh and field points given in the
%       N x 3 matrix POINTS. The resulting matrices are of type N x E,
%       where N is the number of field points (number of rows in POINTS)
%       and E is the number of elements in the mesh.
%       
%   The output matrix D contains the absolute distances, while matrix R 
%   contains the relative distances such that element R(i,j) is the 
%   relative distance to point i from element j, normalized by the 
%   estimated linear size of element j.

% Calculate mesh centers and element sizes
[c, ~, A] = mesh_element_props(mesh);
e = sqrt(2*A);

if (nargin < 2)
    points = c;
end

n_elements = size(mesh.elements, 1);
n_points = size(points, 1);
d = zeros(n_points, n_elements);
for i = 1 : 3
    d = d + bsxfun(@minus, points(:, i), c(:, i).').^2;
end
d = sqrt(d);

% Divide by the element sizes
r = bsxfun(@times, d, 1./e.');

end % of function mesh_distances
