function [xp, nx, W] = mesh_quadrature_points(mesh, order)
%MESH_QUADRATURE_POINTS Create quadrature points over a triangular mesh
%   [XP, NX, W] = MESH_QUADRATURE_POINTS(MESH, ORDER) Creates quadrature
%       points of order ORDER over a triangular mesh MESH for numerical 
%       integration. If ORDER is a scalar, the same order is used for all
%       the elements. Alternatively, ORDER can be a vector having the same
%       number of elements as the mesh. In this case, the order is
%       interpreted element-by-element. If an order of NAN is specified for
%       an element, then no quadrature is computed for that element.
%   Output:
%       XP: N x 3 matrix with N denoting the total number of base points, 
%       containing the base points of the quadratures.
%       NX: N x 3 matrix containing the normal vector at each base point.
%       W : E x N sparse matrix with E denoting the number of elements in
%           the mesh. Evaluating a function in points XP and multiplying
%           the result by W results in a column vector with E elements
%           containing the contribution of each element in the integral.

trias = mesh.elements;
n_trias = size(trias, 1);

% Chek input
if (~isscalar(order) && numel(order) ~= n_trias)
    error('mesh_quadrature_points:invalid_argument', ...
        'Input parameter ORDER must be scalar or have as many elements as the mesh');
end

% Expand scalar orders
if (isscalar(order))
    order = order * ones(n_trias, 1);
end

% Extract x,y,z coordinates of the elements
X = mesh.nodes(:,1);
Y = mesh.nodes(:,2);
Z = mesh.nodes(:,3);

% Extract element areas and normal vectors
[~, n, A] = mesh_element_props(mesh);

% Go order by order
points = ceil((order+1)/2).^2;          % Number of points for each element
n_points = sum(points(~isnan(points))); % The total number of points
u_orders = sort(unique(order));         % Unique order
u_orders = u_orders(~isnan(u_orders));  % Drop NaN from unique orders
u_points = ceil((u_orders+1)/2).^2;     % Unique number of points
n_orders = length(u_orders);            % Number of unique orders

% Preallocate for the results
I = nan(n_points, 1);                   % Row indices of sparse matrix
J = nan(n_points, 1);                   % Column indices of sparse matrix
w = nan(n_points, 1);                   % Values of sparse matrix

xp = nan(n_points, 3);                  % Quadrature base points
nx = nan(n_points, 3);                  % Normal vectors

k = 0;
for i_order = 1 : n_orders
    % Find the elements over which the order is chosen
    sel = find(order == u_orders(i_order));
    % Row indices of the weight matrix is the element index
    rows = kron(sel, ones(u_points(i_order), 1));
    % Column indices of the weight matrix is the quadrature point index
    cols = k + (1 : length(sel) * u_points(i_order)).';

    x = X(trias(sel,:));        % vertices
    y = Y(trias(sel,:));
    z = Z(trias(sel,:));
    % check if there was only one element
    if size(x,2) == 1
       x = x.';
       y = y.';
       z = z.';
    end

    % Compute a quadrature
    [xx, ww] = gaussian_quadrature_2d(u_orders(i_order), 3);
    N = elem_shape_functions(xx, 3);
    gx = N * x.';
    gy = N * y.';
    gz = N * z.';
    % Collect the points
    xp(cols, :) = [gx(:) gy(:) gz(:)];
    % Repeat normal over points in the same element
    nx(cols, :) = kron(n(sel, :), ones(u_points(i_order), 1));
    % Apply element size in weighting
    ww = kron(2*A(sel, :), ww);

    % Store 
    I(cols) = rows;
    J(cols) = cols;
    w(cols) = ww;

    % Step the indices
    k = k + length(sel) * u_points(i_order);
end

% Assemble weight matrix
W = sparse(I, J, w, n_trias, n_points);

end % of function mesh_quadrature_points
