function [nodes, elements] = mesh_select(mesh, expression, selmode)
%MESH_SELECT Element and node selection in a triangular mesh
%   [NODES, ELEMENTS] = MESH_SELECT(MESH, COND, SELMODE) Selects nodes and 
%       elements of the triangular mesh MESH satisfying the condition COND,
%       using selection mode defined by the parameter SELMODE.
%   MESH : Triangular mesh structure
%   COND : Valid Matlab expression string containing the variables
%          'x', 'y', 'z', 'r', 'phi', 'theta', 'R'. These variables refer 
%          to the nodal coordinats as follows:
%          'x'     = MESH.nodes(:,1)
%          'y'     = MESH.nodes(:,2)
%          'z'     = MESH.nodes(:,3)
%          'r'     = sqrt(x^2 + y^2), 2D radius
%          'phi'   = atan2(y, x), 2D angle
%          'theta' = atan2(z, r), 3D inclination angle
%          'R'     = sqrt(x^2 + y^2 + z^2), 3D radius
%   SELMODE : 'all' -> An element is selected if all of its nodes satisfy
%                      the condition (default).
%             'any' -> An element is selected if any of its nodes satisfy
%                      the condition.
%             'none'-> An element is selected if none of its nodes satisfy
%                      the condition.
%             'nall'-> An element is selected if not all of its nodes
%                      satisfy the condition.
%
% See also: 
%   mesh_section

% Default parameters
if nargin < 3
    selmode = 'all';
end

% Defining the expression variables
x = mesh.nodes(:,1);
y = mesh.nodes(:,2);
z = mesh.nodes(:,3);
r = sqrt(x.^2 + y.^2); 
theta = atan2(z,r);             %#ok<NASGU>
R = sqrt(x.^2 + y.^2 + z.^2);   %#ok<NASGU>
phi = atan2(y,x);               %#ok<NASGU>

% Expression evaluation
nElements = size(mesh.elements,1);
% find node indices satisfying the condition
if ischar(expression)
    nodeind = find(eval(expression));
else
    error('mesh_select:invalid_expression', 'Invalid expression');
end
switch lower(selmode)
    case 'any'
        elemind = find(any(ismember(mesh.elements, nodeind),2));
    case 'none'
        elemind = find(any(ismember(mesh.elements, nodeind),2));
        elemind = setdiff(1:nElements,elemind);
    case 'all'
        elemind = find(sum(ismember(mesh.elements, nodeind),2) ==...
            sum(mesh.elements ~=0, 2));
    case 'nall'
        elemind = find(sum(ismember(mesh.elements, nodeind),2) ==...
            sum(mesh.elements ~=0, 2));
        elemind = setdiff(1:nElements,elemind);
    otherwise
        error('mesh_select:invalid_argument', ...
            'Invalid selection mode selector: %s', selmode);
end

nodes = nodeind;
elements = elemind;

end % of function mesh_select
