function varargout = plot_mesh(mesh, varargin)
%PLOT_MESH Plot colored triangular mesh
%   PLOT_MESH(MESH) Plots a triangular mesh. 
%   PLOT_MESH(MESH, 'node', DATA) plots a NiHu mesh colored by DATA
%    containing the color indices of the MESH's nodes.
% PLOT_MESH(MESH, 'elem', DATA) plots a NiHu mesh colored by DATA
%    containing the color indices of the MESH's elements.
% PLOT_MESH(MESH, DATA) plots a colored NiHU mesh. If DATA has the same
%    number of elements as MESH.Nodes, then the coloring is performed
%    nodewise. If DATA has the same number of elements as MESH.Elements,
%    then the coloring is performed elementwise. 
% PLOT_MESH(MESH, 'node', NodeID, DATA) plots a colored NiHU mesh based
%    on nodal color values. NodeID and DATA are Mx1 vectors containing
%    the colored nodes' IDs and color indices.
% PLOT_MESH(MESH, 'elem', NodeID, DATA) plots a colored NiHU mesh based
%    on element color values. ElemID and DATA are Mx1 vectors containing
%    the colored elements' IDs and color indices.

% Argument check and default parameters
narginchk(1, 4);
nargoutchk(0, 1);

switch nargin
    case 1  % PLOT_MESH(MESH);
        cData = zeros(size(mesh.nodes,1),1);
        cDOF = (1 : size(mesh.nodes, 1)).';
        nodewise = true;
    case 2  % PLOT_MESH(MESH, DATA);
        cData = varargin{1};
        if length(cData) == size(mesh.nodes,1)
            cDOF = (1 : size(mesh.nodes, 1)).';
            nodewise = true;
        elseif length(cData) == size(mesh.elements,1)
             cDOF = (1 : size(mesh.elements, 1)).';
            nodewise = false;
        else
            error('plot_mesh:invalid_argument', ...
                'Plot mode can not be determined from length(DATA).');
        end
    case 3
        switch lower(varargin{1})
            case 'node' % PLOT_MESH(MESH, 'node', DATA)
                nodewise = true;
                cDOF = (1 : size(mesh.nodes, 1)).';
            case 'elem' % PLOT_MESH(MESH, 'elem', DATA)
                nodewise = false;
                cDOF = (1 : size(mesh.elements, 1)).';
            otherwise
                error('plot_mesh:invalid_argument', ...
                    'Invalid type specifier ''%s''', varargin{1});
        end
        cData = varargin{2};
    case 4
        switch lower(varargin{1})
            case 'node'  % PLOT_MESH(MESH, 'node', DOF, DATA)
                nodewise = true;
            case 'elem'  % PLOT_MESH(MESH, 'elem', DOF, DATA)
                nodewise = false;
            otherwise
                error('nihu:plot_mesh:argError', ...
                    'Invalid type specifier ''%s''', varargin{1});
        end
        cDOF = varargin{2};
        cData = varargin{3};
end

if numel(cData) ~= numel(cDOF)
    error('nihu:plot_mesh:argError', 'Invalid size of color data');
end

% Convert ID-s to element and node indices, and drop IDs
if nodewise
    ID = (1 : size(mesh.nodes,1)).';
else
    ID = (1 : size(mesh.elements,1)).';
end

% check whether all IDs are contained in the mesh
d = ismember(cDOF, ID);
if ~all(d)
    error('plot_mesh:invalid_ID',...
        ['Invalid IDs (do not refer to mesh IDs):\n'...
        sprintf('%d\n', cDOF(~d))]);
end

dind(ID) = 1:length(ID);    % create ID-to-index permutation vector
c = zeros(size(ID));        % null data
c(dind(cDOF(d))) = cData(d);

x = mesh.nodes(:,1);
y = mesh.nodes(:,2);
z = mesh.nodes(:,3);
elem = mesh.elements;

% Plot triangular elements
if nodewise
    h = patch('Faces', elem,...
        'Vertices', [x y z], ...
        'FaceVertexCData', c,...
        'FaceColor', 'interp');
else
    h = patch('Faces', elem,...
        'Vertices', [x y z], ...
        'FaceVertexCData', c,...
        'FaceColor', 'flat');
end

% Axis settings
xlabel('x');
ylabel('y');
zlabel('z');
axis equal tight;
grid on;
view(3);

if nargout > 0
    varargout{1} = h;
end

end % of function plot_mesh
