% transparent_sphere_test.m
% Exterior radiaton of a sphere with transparent geometry.
%
% A virtual source is set up in the interior part of our geometry. The 
% velocity field of the virtual source is set as the boundary condition on
% the geometry. Then, the exterior field radiated by the body with 

clear;

%% Import the mesh and display it
mesh = gmsh_import_mesh('../meshes/sphere.msh');
params = gmsh_import_params('../meshes/sphere.geo');
plot_mesh(mesh); hold on;
plot_elem_normals(mesh);
%% Create a field mesh and display it too
field = mesh_create_slab([params.r*1.1 0; params.r*3.5 0; 
    params.r*3.5 pi/3; params.r*1.1 pi/3], [50 50]);
field.nodes = [ field.nodes(:,1).*cos(field.nodes(:,2)), ...
                zeros(size(field.nodes, 1), 1), ...
                field.nodes(:,1).*sin(field.nodes(:,2)) ];
plot_mesh(field);            
%% Incident field on surface
% Choose a wave number
k = min(mesh_kmax(mesh, 8));
% Put a virtual point source inside the sphere and compute its field
x_src = [-.5 .3 0]; 
[centers, normals] = mesh_element_props(mesh);
% The pressure on the surface is also the analytical solution
[ps_ana, qs_ana] = incident_field('point', x_src, centers, normals, k);
%% Compute the BEM solution
% Assemble the system matrices
fprintf('BEM surface assembly ... '); tic;
[Gs, Hs] = bem_matrices(k, mesh);
fprintf('Ready in %.2f s.\n', toc);
% And solve the equation
fprintf('Solving BEM system ... '); tic; 
ps_bem = (Hs - 0.5*eye(size(Hs, 1))) \ (Gs * qs_ana);
fprintf('Ready in %.2f s.\n', toc);
% Compute the error
err_s = norm(ps_ana - ps_bem) / norm(ps_ana);
fprintf('Relative error on the surface: %.2f%%\n', err_s * 100);
%% Plot the surface solution
figure;
subplot(1, 2, 1);
plot_mesh(mesh, real(ps_ana));
cb = colorbar('SouthOutside');
xlabel(cb, 'Real part of pressure');
title('Analytical solution');
subplot(1, 2, 2);
plot_mesh(mesh, real(ps_bem));
cb = colorbar('SouthOutside');
xlabel(cb, 'Real part of pressure');
title('BEM solution');
%% Compute the radiated field
pts = field.nodes;
pf_ana = incident_field('point', x_src, pts, [], k);
fprintf('BEM field assembly ... '); tic;
[Gf, Hf] = bem_matrices(k, mesh, pts);
fprintf('Ready in %.2f s.\n', toc);
fprintf('Computing BEM field ... '); tic;
pf_bem = Hf * ps_bem - Gf * qs_ana;
fprintf('Ready in %.2f s.\n', toc);
% Compute the error
err_s = norm(pf_ana - pf_bem) / norm(pf_ana);
fprintf('Relative error on the field: %.2f%%\n', err_s * 100);
%% Plot the field
figure;
plot_mesh(mesh, real(ps_bem)); hold on;
plot_mesh(field, real(pf_bem));
cb = colorbar('SouthOutside');
xlabel(cb, 'Real part of pressure');
title('BEM solution on surface and field');
